"""
Type annotations for botocore.docs.params module.

Copyright 2025 Vlad Emelianov
"""

from collections.abc import Sequence
from typing import Any

from botocore.docs.bcdoc.restdoc import DocumentStructure
from botocore.docs.shape import ShapeDocumenter
from botocore.model import Shape

class BaseParamsDocumenter(ShapeDocumenter):
    def document_params(
        self,
        section: DocumentStructure,
        shape: Shape,
        include: Sequence[str] | None = ...,
        exclude: Sequence[str] | None = ...,
    ) -> None: ...
    def document_recursive_shape(
        self, section: DocumentStructure, shape: Shape, **kwargs: Any
    ) -> None: ...
    def document_shape_default(
        self,
        section: DocumentStructure,
        shape: Shape,
        history: Any,
        include: Sequence[str] | None = ...,
        exclude: Sequence[str] | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def document_shape_type_list(
        self,
        section: DocumentStructure,
        shape: Shape,
        history: Any,
        include: Sequence[str] | None = ...,
        exclude: Sequence[str] | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def document_shape_type_map(
        self,
        section: DocumentStructure,
        shape: Shape,
        history: Any,
        include: Sequence[str] | None = ...,
        exclude: Sequence[str] | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def document_shape_type_structure(
        self,
        section: DocumentStructure,
        shape: Shape,
        history: Any,
        include: Sequence[str] | None = ...,
        exclude: Sequence[str] | None = ...,
        name: str | None = ...,
        **kwargs: Any,
    ) -> None: ...

class ResponseParamsDocumenter(BaseParamsDocumenter):
    EVENT_NAME: str = ...
    def document_shape_type_event_stream(
        self,
        section: DocumentStructure,
        shape: Shape,
        history: Any,
        **kwargs: Any,
    ) -> None: ...

class RequestParamsDocumenter(BaseParamsDocumenter):
    EVENT_NAME: str = ...
    def document_shape_type_structure(  # type: ignore[override]
        self,
        section: DocumentStructure,
        shape: Shape,
        history: Any,
        include: Sequence[str] | None = ...,
        exclude: Sequence[str] | None = ...,
        **kwargs: Any,
    ) -> None: ...



"""
Type annotations for botocore.tokens module.

Copyright 2025 Vlad Emelianov
"""

import datetime
import logging
from collections.abc import Iterable, Mapping
from typing import Any, Callable, NamedTuple

from botocore.session import Session
from botocore.utils import JSONFileCache

logger: logging.Logger

def create_token_resolver(session: Session) -> TokenProviderChain: ...

class FrozenAuthToken(NamedTuple):
    token: str
    expiration: datetime.datetime | None = ...

class DeferredRefreshableToken:
    def __init__(
        self,
        method: Any,
        refresh_using: Callable[[], FrozenAuthToken],
        time_fetcher: Callable[[], datetime.datetime] = ...,
    ) -> None: ...
    def get_frozen_token(self) -> FrozenAuthToken: ...

class TokenProviderChain:
    def __init__(self, providers: Iterable[Any] | None = ...) -> None: ...
    def load_token(self) -> DeferredRefreshableToken: ...

class SSOTokenProvider:
    METHOD: str = ...
    DEFAULT_CACHE_CLS: type[JSONFileCache] = ...

    def __init__(
        self,
        session: Session,
        cache: JSONFileCache | None = ...,
        time_fetcher: Callable[[], datetime.datetime] = ...,
        profile_name: str | None = ...,
    ) -> None: ...
    def load_token(self) -> DeferredRefreshableToken: ...

class ScopedEnvTokenProvider:
    METHOD: str = ...
    def __init__(self, session: Session, environ: Mapping[str, Any] | None = ...) -> None: ...
    def load_token(
        self, *, signing_name: str | None = ..., **kwargs: Any
    ) -> FrozenAuthToken | None: ...



"""
Type annotations for botocore.parsers module.

Copyright 2025 Vlad Emelianov
"""

from collections.abc import Mapping
from io import BufferedReader
from logging import Logger
from typing import IO, Any, Callable

from botocore.compat import XMLParseError as XMLParseError
from botocore.eventstream import EventStream as EventStream
from botocore.eventstream import NoInitialResponseError as NoInitialResponseError
from botocore.model import Shape
from botocore.utils import CachedProperty
from botocore.utils import is_json_value_header as is_json_value_header
from botocore.utils import lowercase_dict as lowercase_dict
from botocore.utils import merge_dicts as merge_dicts
from botocore.utils import parse_timestamp as parse_timestamp

LOG: Logger = ...
DEFAULT_TIMESTAMP_PARSER = parse_timestamp

class ResponseParserFactory:
    def __init__(self) -> None: ...
    def set_parser_defaults(self, **kwargs: Any) -> None: ...
    def create_parser(self, protocol_name: str) -> ResponseParser: ...

def create_parser(protocol: str) -> ResponseParser: ...

class ResponseParserError(Exception): ...

class ResponseParser:
    DEFAULT_ENCODING: str = ...
    EVENT_STREAM_PARSER_CLS: type[ResponseParser] | None = ...
    KNOWN_LOCATIONS: tuple[str, ...] = ...
    def __init__(
        self,
        timestamp_parser: Callable[[str], Any] | None = ...,
        blob_parser: Callable[[str], Any] | None = ...,
    ) -> None: ...
    def parse(self, response: Mapping[str, Any], shape: Shape) -> Any: ...

class BaseXMLResponseParser(ResponseParser):
    def __init__(
        self,
        timestamp_parser: Callable[[str], Any] | None = ...,
        blob_parser: Callable[[str], Any] | None = ...,
    ) -> None: ...

class QueryParser(BaseXMLResponseParser): ...
class EC2QueryParser(QueryParser): ...
class BaseJSONParser(ResponseParser): ...

class BaseCBORParser(ResponseParser):
    INDEFINITE_ITEM_ADDITIONAL_INFO: int = ...
    BREAK_CODE: int = ...
    @CachedProperty
    def major_type_to_parsing_method_map(self) -> dict[int, Callable[[IO[bytes], int], Any]]: ...
    def get_peekable_stream_from_bytes(self, bytes: bytes) -> BufferedReader: ...
    def parse_data_item(self, stream: IO[bytes]) -> Any: ...

class BaseEventStreamParser(ResponseParser): ...
class EventStreamJSONParser(BaseEventStreamParser, BaseJSONParser): ...
class EventStreamXMLParser(BaseEventStreamParser, BaseXMLResponseParser): ...
class EventStreamCBORParser(BaseEventStreamParser, BaseCBORParser): ...
class JSONParser(BaseJSONParser): ...
class BaseRestParser(ResponseParser): ...
class BaseRpcV2Parser(ResponseParser): ...
class RestJSONParser(BaseRestParser, BaseJSONParser): ...
class RpcV2CBORParser(BaseRpcV2Parser, BaseCBORParser): ...
class RestXMLParser(BaseRestParser, BaseXMLResponseParser): ...

PROTOCOL_PARSERS: dict[str, ResponseParser]



"""
Type annotations for botocore.handlers module.

Copyright 2025 Vlad Emelianov
"""

from collections.abc import Mapping
from logging import Logger
from re import Pattern
from typing import Any
from urllib.request import Request

from botocore.client import BaseClient
from botocore.compat import MD5_AVAILABLE as MD5_AVAILABLE
from botocore.compat import OrderedDict as OrderedDict
from botocore.compat import XMLParseError as XMLParseError
from botocore.compat import ensure_bytes as ensure_bytes
from botocore.compat import get_md5 as get_md5
from botocore.compat import unquote as unquote
from botocore.compat import unquote_str as unquote_str
from botocore.compat import urlsplit as urlsplit
from botocore.compat import urlunsplit as urlunsplit
from botocore.docs.bcdoc.restdoc import DocumentStructure
from botocore.exceptions import AliasConflictParameterError as AliasConflictParameterError
from botocore.exceptions import MissingServiceIdError as MissingServiceIdError
from botocore.exceptions import ParamValidationError as ParamValidationError
from botocore.exceptions import UnsupportedTLSVersionWarning as UnsupportedTLSVersionWarning
from botocore.model import OperationModel
from botocore.signers import add_generate_db_auth_token as add_generate_db_auth_token
from botocore.signers import add_generate_presigned_post as add_generate_presigned_post
from botocore.signers import add_generate_presigned_url as add_generate_presigned_url
from botocore.utils import SAFE_CHARS as SAFE_CHARS
from botocore.utils import hyphenize_service_id as hyphenize_service_id
from botocore.utils import percent_encode as percent_encode
from botocore.utils import switch_host_with_param as switch_host_with_param

logger: Logger = ...

REGISTER_FIRST: object
REGISTER_LAST: object
VALID_BUCKET: Pattern[str]
VALID_S3_ARN: Pattern[str]
S3_SIGNING_NAMES: tuple[str, ...]
VERSION_ID_SUFFIX: Pattern[str]
SERVICE_NAME_ALIASES: dict[str, str]

def handle_service_name_alias(service_name: str, **kwargs: Any) -> Any: ...
def add_recursion_detection_header(params: Mapping[str, Any], **kwargs: Any) -> None: ...
def escape_xml_payload(params: Mapping[str, Any], **kwargs: Any) -> Any: ...
def check_for_200_error(response: Any, **kwargs: Any) -> None: ...
def set_operation_specific_signer(context: Any, signing_name: str, **kwargs: Any) -> Any: ...
def decode_console_output(parsed: Mapping[str, Any], **kwargs: Any) -> None: ...
def generate_idempotent_uuid(
    params: Mapping[str, Any], model: OperationModel, **kwargs: Any
) -> None: ...
def decode_quoted_jsondoc(value: Any) -> Any: ...
def json_decode_template_body(parsed: Mapping[str, Any], **kwargs: Any) -> None: ...
def validate_bucket_name(params: Mapping[str, Any], **kwargs: Any) -> None: ...
def sse_md5(params: Mapping[str, Any], **kwargs: Any) -> None: ...
def copy_source_sse_md5(params: Mapping[str, Any], **kwargs: Any) -> None: ...
def disable_signing(**kwargs: Any) -> Any: ...
def add_expect_header(model: OperationModel, params: Mapping[str, Any], **kwargs: Any) -> None: ...

class DeprecatedServiceDocumenter:
    def __init__(self, replacement_service_name: str) -> None: ...
    def inject_deprecation_notice(
        self, section: DocumentStructure, event_name: str, **kwargs: Any
    ) -> None: ...

def document_copy_source_form(
    section: DocumentStructure, event_name: str, **kwargs: Any
) -> None: ...
def handle_copy_source_param(params: Mapping[str, Any], **kwargs: Any) -> None: ...
def inject_presigned_url_ec2(
    params: Mapping[str, Any], request_signer: Any, model: OperationModel, **kwargs: Any
) -> None: ...
def inject_presigned_url_rds(
    params: Mapping[str, Any], request_signer: Any, model: OperationModel, **kwargs: Any
) -> None: ...
def json_decode_policies(
    parsed: Mapping[str, Any], model: OperationModel, **kwargs: Any
) -> None: ...
def parse_get_bucket_location(
    parsed: Mapping[str, Any], http_response: Any, **kwargs: Any
) -> None: ...
def base64_encode_user_data(params: Mapping[str, Any], **kwargs: Any) -> None: ...
def document_base64_encoding(param: Mapping[str, Any]) -> Any: ...
def validate_ascii_metadata(params: Mapping[str, Any], **kwargs: Any) -> None: ...
def fix_route53_ids(params: Mapping[str, Any], model: OperationModel, **kwargs: Any) -> None: ...
def inject_account_id(params: Mapping[str, Any], **kwargs: Any) -> None: ...
def add_glacier_version(
    model: OperationModel, params: Mapping[str, Any], **kwargs: Any
) -> None: ...
def add_accept_header(model: OperationModel, params: Mapping[str, Any], **kwargs: Any) -> None: ...
def add_glacier_checksums(params: Mapping[str, Any], **kwargs: Any) -> None: ...
def document_glacier_tree_hash_checksum() -> Any: ...
def document_cloudformation_get_template_return_type(
    section: DocumentStructure, event_name: str, **kwargs: Any
) -> None: ...
def switch_host_machinelearning(request: Any, **kwargs: Any) -> None: ...
def check_openssl_supports_tls_version_1_2(**kwargs: Any) -> None: ...
def change_get_to_post(request: Request, **kwargs: Any) -> None: ...
def set_list_objects_encoding_type_url(
    params: Mapping[str, Any], context: Any, **kwargs: Any
) -> None: ...
def decode_list_object(parsed: Mapping[str, Any], context: Any, **kwargs: Any) -> None: ...
def decode_list_object_v2(parsed: Mapping[str, Any], context: Any, **kwargs: Any) -> None: ...
def decode_list_object_versions(parsed: Mapping[str, Any], context: Any, **kwargs: Any) -> None: ...
def convert_body_to_file_like_object(params: Mapping[str, Any], **kwargs: Any) -> None: ...

class ParameterAlias:
    def __init__(self, original_name: str, alias_name: str) -> None: ...
    def alias_parameter_in_call(
        self, params: Mapping[str, Any], model: OperationModel, **kwargs: Any
    ) -> None: ...
    def alias_parameter_in_documentation(
        self, event_name: str, section: DocumentStructure, **kwargs: Any
    ) -> None: ...

class ClientMethodAlias:
    def __init__(self, actual_name: str) -> None: ...
    def __call__(self, client: BaseClient, **kwargs: Any) -> Any: ...

class HeaderToHostHoister:
    def __init__(self, header_name: str) -> None: ...
    def hoist(self, params: Mapping[str, Any], **kwargs: Any) -> None: ...

def inject_api_version_header_if_needed(
    model: Any, params: Mapping[str, Any], **kwargs: Any
) -> None: ...
def remove_lex_v2_start_conversation(
    class_attributes: Mapping[str, Any], **kwargs: Any
) -> None: ...
def remove_qbusiness_chat(class_attributes: Mapping[str, Any], **kwargs: Any) -> None: ...
def remove_bedrock_runtime_invoke_model_with_bidirectional_stream(
    class_attributes: Mapping[str, Any], **kwargs: Any
) -> None: ...
def enable_millisecond_timestamp_precision(
    serializer_kwargs: Mapping[str, Any], **kwargs: Any
) -> None: ...
def add_retry_headers(request: Request, **kwargs: Any) -> None: ...
def remove_bucket_from_url_paths_from_model(
    params: Mapping[str, Any], model: OperationModel, context: Mapping[str, Any], **kwargs: Any
) -> None: ...
def remove_accid_host_prefix_from_model(
    params: Mapping[str, Any], model: OperationModel, context: Mapping[str, Any], **kwargs: Any
) -> None: ...
def remove_arn_from_signing_path(request: Request, **kwargs: Any) -> None: ...
def customize_endpoint_resolver_builtins(
    builtins: Mapping[str, Any],
    model: OperationModel,
    params: Mapping[str, Any],
    context: Mapping[str, Any],
    **kwargs: Any,
) -> None: ...
def remove_content_type_header_for_presigning(request: Request, **kwargs: Any) -> None: ...
def handle_expires_header(
    operation_model: OperationModel,
    response_dict: Mapping[str, Any],
    customized_response_dict: dict[str, Any],
    **kwargs: Any,
) -> None: ...
def document_expires_shape(section: DocumentStructure, event_name: str, **kwargs: Any) -> None: ...
def get_bearer_auth_supported_services() -> set[str]: ...

BUILTIN_HANDLERS: list[tuple[Any, ...]] = ...



"""
Type annotations for botocore.plugin module.

Copyright 2025 Vlad Emelianov
"""

from collections.abc import Mapping
from contextvars import Token
from dataclasses import dataclass
from logging import Logger

from botocore.client import BaseClient

log: Logger = ...

@dataclass
class PluginContext:
    plugins: str | None = ...

def get_plugin_context() -> PluginContext | None: ...
def set_plugin_context(ctx: PluginContext) -> Token[PluginContext]: ...
def reset_plugin_context(token: Token[PluginContext]) -> None: ...
def get_botocore_plugins() -> str | None: ...
def load_client_plugins(client: BaseClient, plugins: Mapping[str, str]) -> None: ...



"""
Type annotations for botocore.docs.bcdoc.restdoc module.

Copyright 2025 Vlad Emelianov
"""

import re
from collections.abc import Iterable, Mapping
from logging import Logger
from typing import Any

DEFAULT_AWS_DOCS_LINK: str = ...
DOCUMENTATION_LINK_REGEX: re.Pattern[str] = ...
LARGE_SECTION_MESSAGE: str = ...
LOG: Logger = ...
SECTION_LINE_LIMIT_CONFIG: dict[str, Any] = ...
SECTION_METHOD_PATH_DEPTH: dict[str, int] = ...

class ReSTDocument:
    def __init__(self, target: str = ...) -> None: ...
    def write(self, content: str) -> None: ...
    def writeln(self, content: str) -> None: ...
    def peek_write(self) -> str: ...
    def pop_write(self) -> None: ...
    def push_write(self, s: str) -> None: ...
    def getvalue(self) -> bytes: ...
    def translate_words(self, words: Iterable[str]) -> list[str]: ...
    def handle_data(self, data: str | None) -> None: ...
    def include_doc_string(self, doc_string: str) -> None: ...
    def remove_last_doc_string(self) -> None: ...

class DocumentStructure(ReSTDocument):
    def __init__(
        self,
        name: str,
        section_names: list[str] | None = ...,
        target: str = ...,
        context: Mapping[str, Any] | None = ...,
    ) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def path(self) -> list[str]: ...
    @path.setter
    def path(self, value: list[str]) -> None: ...
    @property
    def available_sections(self) -> list[str]: ...
    @property
    def context(self) -> dict[str, Any]: ...
    def add_new_section(
        self, name: str, context: Mapping[str, Any] | None = ...
    ) -> DocumentStructure: ...
    def get_section(self, name: str) -> DocumentStructure: ...
    def has_section(self, name: str) -> bool: ...
    def delete_section(self, name: str) -> None: ...
    def flush_structure(self, docs_link: str | None = ...) -> bytes: ...
    def getvalue(self) -> bytes: ...
    def remove_all_sections(self) -> None: ...
    def clear_text(self) -> None: ...
    def add_title_section(self, title: str) -> DocumentStructure: ...
    def write_to_file(self, full_path: str, file_name: str) -> None: ...



"""
Type annotations for botocore.translate module.

Copyright 2025 Vlad Emelianov
"""

from collections.abc import Mapping
from typing import Any

from botocore.utils import merge_dicts as merge_dicts

def build_retry_config(
    endpoint_prefix: str,
    retry_model: Mapping[str, Any],
    definitions: Mapping[str, Any],
    client_retry_config: Mapping[str, Any] | None = ...,
) -> Any: ...
def resolve_references(config: Mapping[str, Any], definitions: Mapping[str, Any]) -> None: ...
